%% function to plot out both points and values using a parallel
%% co-ordinate type of presentation.
%%
%% Written by Eric S Fraga, 2005.08.14

%% args are the points x, the lower, a, and upper bounds, b, on x,
%% the objective function values, y, and the file to output the pcs data
%% points to, pcsfile.  x and y are one point per row
function pcs(x, a, b, y, ylimits, pcsfile)
  global plotfront

  persistent version;
  if (isempty(version))
    version = '2013.06.18 18:38:35';
    fprintf ('PCS data generation, version %s\n', version)
  end

  z = [x, y];                   % put both points and values together
  m = size(z, 2);               % number of columns: co-ordinates and criteria
  n = size(z, 1);               % number of rows: number of points
  %% we wish to scale all objective function values to fit within [0,1]
  maxyValues = ylimits(2)
  minyValues = ylimits(1)
  yr = maxyValues - minyValues;
  zeroRange = find( abs(yr)<1e-4 );
  if size(zeroRange) > 0 
    yr(zeroRange) = 1e-4;
  end
  %% the actual range of values is a concatenation of [a,b] and yr
  minValues = [a, minyValues];
  maxValues = [b, maxyValues];
  r = [(b-a), yr];
  %% create the data to be plotted consisting of a line for each point
  fid = fopen(pcsfile, 'wt', 'native')
  fprintf(fid, '%% Generated by PCS.M code, %s', ctime(time()));
  for i=1:n
    for j=1:m
      normalised = (z(i,j)-minValues(j))/r(j);
      if normalised < 0 || normalised > 1.0
        fprintf('data point %d:%d is out of range [%g,%g]\n', i, j, minValues(j), maxValues(j));
      end
      fprintf(fid, '%8d %13.6g\n', j, normalised );
    end
    fprintf(fid, '\n');
  end
  fclose(fid);
end
