function [fit, best] = strawberry_fitness(p,nx,nz,npop)
  global strawberry_fitness_method
  [n m] = size(p);
  debugprint (1, 'fitness, nx nz m n npop', [nx, nz, m, n, npop])
  if m <= 0
    disp('No solutions in the population');
  end
  fit = zeros(m,1);
  factor = 1;                % for placement in fitness interval (0,1)
  indexfeasible = p(n,:) <= 0;
  nfeas = sum(indexfeasible);
  indexinfeasible = p(n,:) > 0;
  ninfeas = sum(indexinfeasible);
  if nfeas > 0
    feasiblefit = strawberry_vectorfitness(nz, p(nx+1:nx+nz,indexfeasible));
    %% the ranking of the feasible solutions is possibly adjusted to
    %% fit in only the upper half of the [0,1] interval if there are
    %% infeasible solutions as well.  The latter will have fitness
    %% values in the bottom half of the interval.
    if ninfeas > 0
      factor = 2;
    end
    fit(indexfeasible) = (feasiblefit+factor-1)/factor;
  end
  if ninfeas > 0
    %% squeeze infeasible fitness values into (0,0.5) or (0,1)
    %% depending on factor, i.e. whether there are any feasible
    %% solutions as well or not
    fit(indexinfeasible) = strawberry_vectorfitness(1, p(n,indexinfeasible))/factor;
  end
  if nz < 2
    [bestfit index] = max(fit);
    best = p(:,index);
  else
    %% fprintf('There are %d feasible solutions when calculating fitness\n',nfeas)
    if nfeas > 0
      allindices = 1:m;
      feasibleindices = allindices(indexfeasible);
      %% fprintf('feasible population:\n')
      %% disp(p(:,indexfeasible))
      %% fprintf('and pareto from that population\n')
      %% disp(p(:,feasibleindices(findpareto(p(nx+1:nx+nz,indexfeasible)))))
      best = p(:,feasibleindices(findpareto(p(nx+1:nx+nz,indexfeasible))));
    else
      best = [];
    end
  end
end
